const std = @import("std");
const zap = @import("zap");

const models = @import("../models/root.zig");
const stores = @import("../stores/root.zig");
const msg_queue = @import("../../msg_queue/root.zig");

const Self = @This();

pub const default_path = "block";

allocator: std.mem.Allocator,
block_update_queue: *const stores.BlockUpdateQueue,

path: []const u8,
error_strategy: zap.Endpoint.ErrorStrategy = .log_to_console,

pub fn init(allocator: std.mem.Allocator, block_update_queue: *const stores.BlockUpdateQueue, path: []const u8) Self {
    return .{
        .allocator = allocator,
        .block_update_queue = block_update_queue,
        .path = path,
    };
}

pub fn get(_: *Self, r: zap.Request) !void {
    r.setStatus(.method_not_allowed);
    r.markAsFinished(true);
}

pub fn post(_: *Self, r: zap.Request) !void {
    r.setStatus(.method_not_allowed);
    r.markAsFinished(true);
}

pub fn put(self: *Self, r: zap.Request) !void {
    blk: {
        if (r.body) |body| {
            const maybe_block: ?std.json.Parsed(models.Block) = std.json.parseFromSlice(models.Block, self.allocator, body, .{}) catch null;
            if (maybe_block) |parsed| {
                defer parsed.deinit();
                const block = parsed.value;
                if (block.material.len > msg_queue.messages.BlockUpdate.material_len_max or !models.Block.materialIsValid(block.material)) {
                    break :blk;
                }
                std.log.info("block: {}", .{block});

                var msg = msg_queue.messages.BlockUpdate{
                    .dimension = block.dimension,
                    .x = block.x,
                    .y = block.y,
                    .z = block.z,
                };
                std.mem.copyForwards(u8, &msg.material, block.material);
                msg.material_len = block.material.len;
                self.block_update_queue.queue.enqueue(msg);

                r.setStatus(.created);
                r.markAsFinished(true);
                return;
            }
        }
    }

    r.setStatus(.bad_request);
    r.markAsFinished(true);
}

pub fn delete(_: *Self, r: zap.Request) !void {
    r.setStatus(.method_not_allowed);
    r.markAsFinished(true);
}

pub fn patch(_: *Self, r: zap.Request) !void {
    r.setStatus(.method_not_allowed);
    r.markAsFinished(true);
}

pub fn options(_: *Self, r: zap.Request) !void {
    try r.setHeader("Access-Control-Allow-Origin", "*");
    try r.setHeader("Access-Control-Allow-Methods", "PUT, OPTIONS, HEAD");
    r.setStatus(zap.http.StatusCode.no_content);
    r.markAsFinished(true);
}

pub fn head(_: *Self, r: zap.Request) !void {
    r.setStatus(zap.http.StatusCode.no_content);
    r.markAsFinished(true);
}
